﻿unit fonctions_vtree;
// Unité de la Version 2 du projet FormMain
// La version 1 TFormMain n'est pas sa fenêtre parente

// Le module crée des propriété servant à la gestion du fichier INI
// Il gère la déconnexion
// Il gère la gestion des touches majuscules et numlock
// Il gère les forms enfants
// créé par Matthieu Giroux en décembre 2007

{$I ..\DLCompilers.inc}
{$I ..\extends.inc}

interface

{$IFDEF FPC}
{$mode Delphi}
{$ENDIF}

uses
     {$IFDEF VERSIONS}
       fonctions_version,
     {$ENDIF}
     {$IFDEF VIRTUALTREES}
        VirtualTrees;
     {$ELSE}
       ComCtrls;
     {$ENDIF}

{$IFDEF VERSIONS}
const
 gVer_fonctions_vtree : T_Version = ( Component : 'Virtual Tree management' ; FileUnit : 'fonctions_vtree' ;
                             			           Owner : 'Matthieu Giroux' ;
                             			           Comment : 'Virtual Tree'' Functions.' ;
                             			           BugsStory : 'Version 1.0.1.0 : Adding comctrls.'+#10
                                                                     + 'Version 1.0.0.1 : Testing getnodelevel.'+#10
                                                                     + 'Version 1.0.0.0 : Working.';
                             			           UnitType : 1 ;
                             			           Major : 1 ; Minor : 0 ; Release : 1 ; Build : 0 );
{$ENDIF}

{$IFNDEF VIRTUALTREES}
type
  // TVTLineType is used during painting a tree
  TVTLineType = (
    ltNone,          // no line at all
    ltBottomRight,   // a line from bottom to the center and from there to the right
    ltTopDown,       // a line from top to bottom
    ltTopDownRight,  // a line from top to bottom and from center to the right
    ltRight,         // a line from center to the right
    ltTopRight,      // a line from bottom to center and from there to the right
    // special styles for alternative drawings of tree lines
    ltLeft,          // a line from top to bottom at the left
    ltLeftBottom     // a combination of ltLeft and a line at the bottom from left to right
  );
  // A collection of line type IDs which is used while painting a node.
  TLineImage = array of TVTLineType;

function GetNodeLevel ( const Node: TTreeNode):Integer;
{$ENDIF}
function NodeIsVisible(const Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;
function HasVisibleNextSibling( Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;
function HasVisiblePreviousSibling( Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;
function IsFirstVisibleChild(Parent, Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;
function IsLastVisibleChild(Parent, Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;


implementation

{ Standard Functions }

{$IFNDEF VIRTUALTREES}
function GetNodeLevel ( const Node: TTreeNode):Integer;
Begin
  Result:=Node.Level;
end;
{$ENDIF}


//   virtual tree sources
function NodeIsVisible(const Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;

// Checks if a node will effectively be hidden as this depends on the nodes state and the paint options.

begin
  if Assigned(Node) then
    Result := {$IFDEF VIRTUALTREES}( vsVisible in Node.States ){$ELSE}Node.Visible{$ENDIF}
  else
    Result := False;
end;


//----------------------------------------------------------------------------------------------------------------------
//   virtual tree sources
function HasVisibleNextSibling( Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;

// Helper method to determine if the given node has a visible next sibling. This is needed to
// draw correct tree lines.

begin
  // Check if there is a sibling at all.
  Result := Assigned(Node.{$IFDEF VIRTUALTREES}NextSibling{$ELSE}GetNextSibling{$ENDIF});

  if Result then
  begin
    repeat
      Node := Node.{$IFDEF VIRTUALTREES}NextSibling{$ELSE}GetNextSibling{$ENDIF};
      Result := NodeIsVisible ( Node );
    until Result or (Node.{$IFDEF VIRTUALTREES}NextSibling{$ELSE}GetNextSibling{$ENDIF} = nil);
  end;
end;


//   virtual tree sources
function HasVisiblePreviousSibling( Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;

// Helper method to determine if the given node has a visible previous sibling. This is needed to
// draw correct tree lines.

begin
  // Check if there is a sibling at all.
  Result := Assigned(Node.{$IFDEF VIRTUALTREES}PrevSibling{$ELSE}GetPrevSibling{$ENDIF});

  if Result then
  begin
    repeat
      Node := Node.{$IFDEF VIRTUALTREES}PrevSibling{$ELSE}GetPrevSibling{$ENDIF};
      Result := NodeIsVisible ( Node );
    until Result or (Node.{$IFDEF VIRTUALTREES}PrevSibling{$ELSE}GetPrevSibling{$ENDIF} = nil);
  end;
end;

//   virtual tree sources
function IsFirstVisibleChild(Parent, Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;

// Helper method to check if Node is the same as the first visible child of Parent.

var
  Run: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};

begin
  // Find first visible child.
  Run := Parent;
  while Assigned(Run) and not (NodeIsVisible ( Node )) do
    Run := Run.{$IFDEF VIRTUALTREES}NextSibling{$ELSE}GetNextSibling{$ENDIF};

  Result := Assigned(Run) and (Run = Node);
end;

function IsLastVisibleChild(Parent, Node: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF}): Boolean;

// Helper method to check if Node is the same as the last visible child of Parent.

var
  Run: {$IFDEF VIRTUALTREES}PVirtualNode{$ELSE}TTreeNode{$ENDIF};

begin
  // Find last visible child.
  Run := Parent.{$IFDEF VIRTUALTREES}LastChild{$ELSE}GetLastChild{$ENDIF};
  while Assigned(Run) and not (NodeIsVisible ( Node )) do
    Run := Run.{$IFDEF VIRTUALTREES}PrevSibling{$ELSE}GetPrevSibling{$ENDIF};

  Result := Assigned(Run) and (Run = Node);
end;


{$IFDEF VERSIONS}
initialization
  p_ConcatVersion ( gVer_fonctions_vtree );
{$ENDIF}
end.
